// ヒートベッド温度コントローラ 押しボタンスイッチ対応版
// 2016年4月26日 伊藤大智

#include "myLCD.h"
#include "ADT7410.h"
#include <Wire.h>

int osmAdrs = 0x48; // 温度センサモジュールのスレーブアドレス
int AnalogPin = 0; // サーミスタ接続ピンの番号
int Heater = 3; // ヒータ接続ピンの番号
int BackLight=13; // LCDのバックライトのピン番号
int STOP_SW = 10; // 白スイッチのピン番号
int PLA_SW = 11; // オレンジスイッチのピン番号
int ABS_SW = 12; // 赤スイッチのピン番号

int PLA_Tr = 70; // PLAモードの設定温度値
int ABS_Tr = 110; // ABSモードの設定温度値
float Kp_def=3.168049; // Kpデフォルト値
float Ki_def=-0.022361;// Kiデフォルト値
int dt = 500; // サンプリングタイム(単位:ms)
int max_power=70; // ヒーター最大出力値（単位:W)

myLCD lcd;
ADT7410 osm;

void setup(void) {
  pinMode(AnalogPin,INPUT); // サーミスタ入力ピン
  pinMode(Heater,OUTPUT); // ヒーターPWM出力ピン
  pinMode(BackLight,OUTPUT); // LCDバックライトピン
  Serial.begin(9600);
  Wire.begin(); // I2Cマスタ初期化 マスタとしてバスに接続  
  lcd.initLcd(); // LCD初期化
  osm.initADT7410(osmAdrs); // 温度センサモジュール初期化
}


void loop(void) {
  
  float osmtmp; // 温度センサモジュールの温度値
  float trmtmp; // サーミスタの温度値
  int  Tr=0; // 温度設定値
  float Ts=0; // 温度初期値
  float Tc=0; // センサと初期温度の差
  float Trc=0; // 目標値と初期温度の差
  float Z=0; // 温度の積分値
  float U=0; // ヒータ出力値(単位:W)
  int duty=0;
  float Kp;
  float Ki;
  int runmode=0; // 0:STOP 1:PLA 2:ABS 3:MANUAL
  int line[5]; // サーミスタADコンバータ値フィルタ用配列
  
  // 最初に5回データ取得
  for(int i=0;i<5;i++){
    trmtmp=thermistor(line); // サーミスタから温度取得
    osmtmp=osm.gettmp(osmAdrs); // 温度センサモジュールから温度取得    
    lcdDisp(0,osmtmp,0,0,0); // LCDに各種情報を表示
    Serial.print(i+1);
    Serial.println(" loading...");
    delay(dt);
  }
  
// 以下無限ループ
while(1){
  
  if(runmode !=0 ){ // 動作モードがSTOP以外だったら    
    Ts=trmtmp; // 温度初期値を設定
    Trc=Tr-Ts; // 目標値と初期温度の差
    Z=0; // 積分値リセット

    Serial.print("Kp = ");
    Serial.println(Kp,6);
    Serial.print("Ki = ");
    Serial.println(Ki,6);
    Serial.println(", Tr,     Ts,    Trc, trmtmp, osmtmp,     Tc,         Z,      U,dty");

    // 以下無限ループ
    while(1){      
      trmtmp=thermistor(line); // サーミスタから温度取得
      osmtmp=osm.gettmp(osmAdrs); // 温度センサモジュールから温度取得
      Tc =trmtmp-Ts; // センサと初期温度の差
      Z  =(-Tc+Trc)*(dt/1000.0)+Z; // 目標値とセンサ温度の差を積分
      U  =-Kp*Tc-Ki*Z; // PI制御でヒーターの出力U[W]を決定

      if(U>max_power){
        U=max_power;
      }
      if(U<0){
        U=0;
      }

      duty=U/max_power*255; // 0からmax_powerの値をとるUの値を、0から255の値をとるduty比に変換
      analogWrite(Heater,duty);

      lcdDisp(trmtmp,osmtmp,runmode,Tr,duty); // LCDに各種情報を表示
      srlDisp(Tr,Ts,Trc,trmtmp,osmtmp,Tc,Z,U,duty); // シリアルで各種情報を送信
      delay(dt); // サンプリングタイム　待つ
      
      if(command_read(&runmode,&Tr,&Kp,&Ki) !=0){
        break; // コマンドが入力されていれば無限ループを抜ける
      }
      if(digitalRead(STOP_SW)==1){
        runmode=0; // STOPモードに変更
        push(runmode,&Tr,&Kp,&Ki); // STOPモードに変更
        break;
      }
    }
  }
  
  else{ // runmodeがSTOPだったら
    Serial.println(", Tr,     Ts,    Trc, trmtmp, osmtmp,     Tc,         Z,      U,dty");
    duty=0;
    analogWrite(Heater,0); // ヒーター出力OFF
    // 以下無限ループ
    while(1){
      trmtmp=thermistor(line); // サーミスタから温度取得
      osmtmp=osm.gettmp(osmAdrs); // 温度センサモジュールから温度取得
      lcdDisp(trmtmp,osmtmp,runmode,Tr,duty); // LCDに各種情報を表示
      srlDisp(Tr,Ts,Trc,trmtmp,osmtmp,Tc,Z,U,duty); // シリアルで各種情報を送信
      delay(dt); // サンプリングタイム　待つ
      
      if(command_read(&runmode,&Tr,&Kp,&Ki) !=0){
        break; // コマンドが入力されていれば無限ループを抜ける
      }
      if(digitalRead(PLA_SW)==1){
        runmode=1; // PLAモードに変更
        push(runmode,&Tr,&Kp,&Ki); // PLAモードに変更
        break;
      }
      if(digitalRead(ABS_SW)==1){
        runmode=2; // ABSモードに変更
        push(runmode,&Tr,&Kp,&Ki); // ABSモードに変更
        break;
      }
    }
  }
}
}


/**********************************************************************************/
// LCDに各種情報を表示する関数
void lcdDisp(float trmtmp, float osmtmp, int runmode, int Tr, int duty){
  char str[16];

  // サーミスタの温度値を表示
  lcd.setCursor(0,1); // 1行目0文字目から
  dtostrf(trmtmp,5,1,str); // 表示文字数5,小数第1位まで表示
  lcd.string(str);
  lcd.string("\xDF\x43"); // ℃
  
  // 温度センサモジュールの温度値を表示
  lcd.setCursor(8,1); // 1行目8文字目から
  dtostrf(osmtmp,6,2,str); // 表示文字数6,小数第2位まで表示
  lcd.string(str);
  lcd.string("\xDF\x43"); // ℃

  // 実行モードを表示
  lcd.setCursor(0,2); // 2行目0文字目から 
  switch(runmode){
    case 0:
      lcd.string("STOP      ");
      break;
    case 1:
      sprintf(str,"PLA(%3d\xDF\x43)",Tr); // PLA(Tr℃)
      lcd.string(str);
      break;
    case 2:
      sprintf(str,"ABS(%3d\xDF\x43)",Tr); // ABS(Tr℃)
      lcd.string(str);
      break;
    case 3:
      sprintf(str,"MAN(%3d\xDF\x43)",Tr); // MAN(Tr℃)
      lcd.string(str);
      break;
  }

  // duty比を表示
  lcd.setCursor(13,2); //2行目13文字目から  
  sprintf(str,"%3d",duty);
  lcd.string(str);

  // バックライトを点灯・消灯
  if(duty>0 || trmtmp>40){ // ヒーターONまたは40度以上のとき
    digitalWrite(BackLight,HIGH);
  }
  else{
    digitalWrite(BackLight,LOW);
  }
}

/**********************************************************************************/
// シリアルで各種情報を送信する関数
void srlDisp(int Tr, float Ts, float Trc, float trmtmp, float osmtmp, float Tc, float Z, float U,int duty){
  char str[16];

  Serial.print(",");
  sprintf(str,"%3d",Tr); // 1番目 設定目標温度値(一定)
  Serial.print(str); 
  Serial.print(",");

  dtostrf(Ts,7,3,str); // 2番目 初期温度値(一定)
  Serial.print(str);
  Serial.print(",");

  dtostrf(Trc,7,3,str); // 3番目 目標値と初期温度の差(一定)
  Serial.print(str);
  Serial.print(",");
  
  dtostrf(trmtmp,7,3,str); // 4番目 サーミスタ温度値
  Serial.print(str); 
  Serial.print(",");
  
  dtostrf(osmtmp,7,3,str); // 5番目 温度センサモジュール温度値
  Serial.print(str);
  Serial.print(",");
  
  dtostrf(Tc,7,3,str); // 6番目 センサと初期温度の差
  Serial.print(str);
  Serial.print(",");
  
  dtostrf(Z,10,3,str); // 7番目 目標値とセンサ温度の差の積分値
  Serial.print(str);
  Serial.print(",");
  
  dtostrf(U,7,3,str); // 8番目 ヒーターの出力U[W]
  Serial.print(str);  
  Serial.print(",");

  sprintf(str,"%3d",duty); // 9番目 ヒーター出力のduty比
  Serial.print(str); 
  Serial.println();
}

/**********************************************************************************/
// サーミスタの温度値を取得する関数
float thermistor(int *line){
  int adc_ans; // ADコンバータの読み取り値
  float adc_fil; // ADコンバータの読み取り値フィルタ処理後
  float Vt; // サーミスタ抵抗値測定用回路の出力電圧値
  float Rn; // サーミスタの抵抗値
  float t; // 温度値（絶対温度K)
  float dmy1,dmy2,dmy3;
  
  // ADコンバータの値を読み取って表示
  adc_ans=analogRead(AnalogPin);
//  Serial.print(","); // 先頭にカンマをつけるとシリアルプロッタが動作しない
//  Serial.print(adc_ans);
//  Serial.print(",");

  // ADコンバータの値にフィルタをかけて表示
  move_line(line,adc_ans); // 直前5個のデータの配列を作る
  adc_fil=line_ave(line); //要素数5の配列の平均値を計算
//  Serial.print(adc_fil);
//  Serial.print(",");

  // 電圧値Vtを計算して表示
  Vt=(5.0*adc_ans)/1024.0; // Vt= 5/1024 * adc_ans  小数点と0をつけるとうまくいった
//  Serial.print(Vt);
//  Serial.print(",");

  // サーミスタの抵抗値を計算して表示
  Rn=adc_fil/(1024.0-adc_fil)*100000; // フィルタ処理後のADコンバータ値を使用する場合
//  Rn=adc_ans/(1024.0-adc_ans)*100000; // フィルタ処理前の生ADコンバータ値を使用する場合 Rn=adc_ans/(1024-adc_ans)*R1
//  Serial.print(Rn);
//  Serial.print(",");

  // サーミスタの温度を計算して表示
//  t=1/(log(Rn/100000)/4100+(1/298.15)); // B定数の式を利用
    // スタインハーツの式を利用
  dmy1=log(Rn/100000);
  dmy2=dmy1*dmy1;
  dmy3=dmy1*dmy2;
  t = 1/( 0.003354016 + 0.0002482770*dmy1 + (1.839899e-6)*dmy2 + (-2.616710e-8)*dmy3 ); // スタインハーツ式
//  Serial.print(t);
//  Serial.print(",");
//  Serial.print(t-273.15); // サーミスタの温度計算結果を表示
//  Serial.print(",");

  return t-273.15;
}

/**********************************************************************************/
// 直前5個のデータの配列を作る関数
void move_line(int *line,int new_adc){
  line[0]=line[1];
  line[1]=line[2];
  line[2]=line[3];
  line[3]=line[4];
  line[4]=new_adc;
}

/**********************************************************************************/
// 要素数5の配列の平均値を計算する関数
float line_ave(int *line){
  int i,sum=0;
  float ave;
  
  for(i=0;i<5;i++){
    sum=sum+line[i];
  }
  ave=sum/5.0;
  return ave;
}

/**********************************************************************************/
// コマンドが入力された場合処理する関数
int command_read(int *runmode, int *Tr, float *Kp, float *Ki){
  char c;
  char buff[16];
  
    if(Serial.available()){ // コマンドが入力されたら
    c=Serial.read();
    Serial.read(); // 改行コード用ダミー
    Serial.read(); // 改行コード用ダミー
    
    if(c=='p'){ // PLAモード
      *runmode=1;
      *Tr=PLA_Tr;
      *Kp=Kp_def;
      *Ki=Ki_def;
      Serial.println("PLA Mode START");
      return 1;
    }
    if(c=='a'){ // ABSモード
      *runmode=2;
      *Tr=ABS_Tr;
      *Kp=Kp_def;
      *Ki=Ki_def;
      Serial.println("ABS Mode START");
      return 1;
    }
    if(c=='m'){ // MANUALモード
      *runmode=3;
      Serial.println("MANUAL Mode Tr Kp Ki SET");
      Serial.println("Tr = ?");
      Serial_gets(buff,'\r'); // Trを入力
      *Tr=atoi(buff);
      Serial.println(*Tr); // Trを表示
      Serial.println("Kp = ?");
      Serial_gets(buff,'\r'); // Kpを入力
      *Kp=atof(buff);
      Serial.println(*Kp,6); // Kpを表示
      Serial.println("Ki = ?");
      Serial_gets(buff,'\r'); // Kiを入力
      *Ki=atof(buff);
      Serial.println(*Ki,6); // Kiを表示
      Serial.println("MANUAL Mode START");
      return 1;
    }
    if(c=='f'){ // バージョン情報表示
      Serial.println("******************************");
      Serial.println("heatbed06.ino");
      Serial.println("Creation Date: 2016/04/26");
      Serial.println("Copyright(c) 2016 Ito Taichi");
      Serial.println("All Right Reserved");
      Serial.println("******************************");
    }
    else{ // STOPモード
      *runmode=0;
      *Tr=0;
      Serial.println("STOP");
      return 1;
    }
  }
  return 0; // コマンドが入力されていなければ0を返す
}

/**********************************************************************************/
/* 特定の文字swがくるまでSerial.readする関数 */
void Serial_gets(char buff[],char sw) {
  int i=0;
  char c;
  while(1) {
    if(Serial.available()) {
      c=Serial.read();
      buff[i]=c;
      if (c==sw) break;
      i++;
    }
  }
buff[i]='\0';
}

/**********************************************************************************/
/* 押されたスイッチの番号に応じて動作モードを変更する関数 */
void push(int runmode, int *Tr, float *Kp, float *Ki){
  
  if(runmode==1){ // PLAモード
    *Tr=PLA_Tr;
    *Kp=Kp_def;
    *Ki=Ki_def;
    Serial.println("PLA Mode START");
  }
  else if(runmode==2){ // ABSモード
    *Tr=ABS_Tr;
    *Kp=Kp_def;
    *Ki=Ki_def;
    Serial.println("ABS Mode START");
  }
  else{ // STOPモード
    *Tr=0;
    Serial.println("STOP");
  }  
}

