// I2C LCD用ライブラリ ソースファイル

#include <arduino.h>
#include <Wire.h>
#include "myLCD.h"


//コンストラクタを定義
myLCD::myLCD()
{  
}

// LCDコマンド送信
void myLCD::writeCmd(byte cmd) {
    byte rs_flg;
    Wire.beginTransmission(LCDadrs); //アドレスのI2Cスレーブに対して送信処理を開始
    rs_flg = 0x00;          // LCDコマンド指定 NULLと認識されないように、変数に入れてbyte型であると明確にする
    Wire.write(rs_flg);     // 第一バイト送信 (コマンド指定)
    Wire.write(cmd);        // 第二バイト送信 ( LCDコマンド)
    Wire.endTransmission(); // ストップ・コンディション発行
}

// LCDデータ送信
void myLCD::writeData(char data) {
    Wire.beginTransmission(LCDadrs);
    Wire.write(0x80);       // 第一バイト送信 (データ指定)
    Wire.write(data);        // 第二バイト送信 (LCDデータ)
    Wire.endTransmission();
}

// LCD初期化
void myLCD::initLcd(void) {
    writeCmd(0x01); // Clear Display
    delay(2); // 動作終了待ち
    writeCmd(0x38); // Function Set (8bit、行数2行、5x8ドット文字に設定）
    writeCmd(0x0C); // Display ON/OFF Control (ディスプレイON、カーソルOFF、カーソル点滅OFF)
    writeCmd(0x06); // Entry Mode Set (カーソル移動方向は右、ディスプレイ全体のシフトは不可)
}

// 文字列の表示(NULL終端の文字列)
void myLCD::string(char *str) {
    byte i;
    for(i = 0; i < 16; i++) {
        if(str[i] == 0x00) {
            break;
        } else {
            writeData(str[i]);
        }
    }
}

// 画面のクリア
void myLCD::clear(void){
    writeCmd(0x01);     // Clear Display
    delay(2);           // 動作終了待ち
}

// カーソル移動
void myLCD::setCursor(byte col, byte row) //col:左から何文字目か row:1行目か2行目か
{
    if ( row > 1 ) { // 2行目の場合
        writeCmd( 0x80 | (col | 0x40)); // 0x80:Set DDRAM Address
    }
    else { // 1行目の場合
        writeCmd( 0x80 | col);
    }
}
